/*
 * modified for lecture by Fabian Fuchs (KIT, Karlsruhe) based on:
 * 
 Copyright (c) 2007, Distributed Computing Group (DCG)
                    ETH Zurich
                    Switzerland
                    dcg.ethz.ch

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

 - Neither the name 'Sinalgo' nor the names of its contributors may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package projects.uebung1.nodes.nodeImplementations;


import java.awt.Color;
import java.awt.Graphics;

import projects.uebung1.nodes.messages.DefaultMessage;
import sinalgo.configuration.Configuration;
import sinalgo.configuration.WrongConfigurationException;
import sinalgo.gui.transformation.PositionTransformation;
import sinalgo.nodes.Node;
import sinalgo.nodes.edges.Edge;
import sinalgo.nodes.messages.Inbox;
import sinalgo.nodes.messages.Message;
import sinalgo.runtime.Main;
import sinalgo.tools.Tools;
import sinalgo.tools.logging.Logging;




/**
 * A leader election node 
 */
public class LeaderElectionNode extends Node {

	/* possible states of the algorithm, possibly more */ 
	public static final int NONLEADER = 0;
	public static final int LEADER = 1;

	/* current state of the algorithm */
	private int state;
	
	/* store the current round */
	private int round;

	/* maximal ID received */
	/* This is already given to allow visualizing the spread of the important maxIDs 
	 * in the draw method. Other required variables may be added. 
	 */
	private int maxID; 
	
	/* add additional variables / flags here */
	
	
	/* payload for message */
	public class LEPayload {
		public int fromID;
		public int maxID;
		public boolean newFollower;
		public LEPayload (int fid, int mid, boolean nf) {
			fromID = fid;
			maxID = mid;
			newFollower = nf;
		}
	}
	
	@Override
	public void handleMessages(Inbox inbox) {
		while(inbox.hasNext()) {
			Message msg = inbox.next();
			if(msg instanceof DefaultMessage<?>) {
				
				//only use LEPayload as payload in DefaultMessage, otherwise this fails
				LEPayload payload = (LEPayload)((DefaultMessage<?>) msg).getLoad();
				
				//handle message here. One option to consider all messages 
				//is to set flags here and handle them in postStep
				
			}
		}
	}

	@Override
	public void preStep() {
		//executed before each 'step' (i.e. timers and handleMessages)
		//I would personally recommend to only use the postStep method for simplicity

	}
	
	@Override
	public void postStep() {
		//executed after each 'step' (i.e. timers and handleMessages)
		/* Hints on how to transmit different messages to different neighbors:
		for (Edge e : this.outgoingConnections) {
			Node n = e.endNode;
			//n.ID gives ID of node
			//Tools.getNodeByID(id) gives Node by ID
			//explore the methods broadcast and send of the class Node 
		}
		*/
		
		round++;
	}

	@Override
	public void init() {
		// initialize the node / read configuration values
		state = NONLEADER;
		round = 0;
		maxID = this.ID;
	}

	@Override
	public void neighborhoodChange() {
		//used / relevant only for initialization and mobility
	}
		
	/* (non-Javadoc)
	 * @see sinalgo.nodes.Node#draw(java.awt.Graphics, sinalgo.gui.transformation.PositionTransformation, boolean)
	 */
	public void draw(Graphics g, PositionTransformation pt, boolean highlight) {
		//set the color of this node, this allows us to see how the most important maxIDs spread
		if (state == LEADER) {
			this.setColor(Color.BLUE); 
		} else {
			int numNodes = Tools.getNodeList().size();
			this.setColor(new Color((float) Math.max(0.0,(float)(maxID-numNodes+5)/5.0), 0, 0));
		}
		String text = Integer.toString(this.ID);
		// draw the node as a circle with the text inside
		super.drawNodeAsDiskWithText(g, pt, highlight, text, 10, Color.YELLOW);
	}


	@Override
	public String toString() {
		return "Node " + this.ID; 
	}

	@Override
	public void checkRequirements() throws WrongConfigurationException {
		//does configuration fit the nodes requirements? 
		
	}
}
